
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.012.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "time tracking for impatient and lazy command line lovers",
  "dist_author" => [
    "Thomas Klausner <domm\@cpan.org>"
  ],
  "dist_name" => "App-TimeTracker",
  "dist_version" => "3.006",
  "license" => "perl",
  "module_name" => "App::TimeTracker",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Data::Dumper" => 0,
    "DateTime" => 0,
    "DateTime::Format::Duration" => 0,
    "DateTime::Format::ISO8601" => 0,
    "Exporter" => 0,
    "File::Copy" => 0,
    "File::Find::Rule" => 0,
    "File::HomeDir" => 0,
    "Hash::Merge" => 0,
    "JSON::XS" => 0,
    "Moose" => 0,
    "Moose::Role" => 0,
    "Moose::Util::TypeConstraints" => 0,
    "MooseX::Getopt" => 0,
    "MooseX::Storage" => 0,
    "MooseX::Storage::Format::JSONpm" => 0,
    "MooseX::Types::Path::Class" => 0,
    "Path::Class" => 0,
    "Path::Class::Iterator" => 0,
    "Scalar::Util" => 0,
    "Term::ANSIColor" => 0,
    "Text::Table" => 0,
    "Try::Tiny" => 0,
    "User::pwent" => 0,
    "constant" => 0,
    "namespace::autoclean" => 0,
    "parent" => 0,
    "perl" => "5.010",
    "strict" => 0,
    "warnings" => 0
  },
  "script_files" => [
    "bin/tracker",
    "bin/tracker_bash_autocomplete"
  ],
  "test_requires" => {
    "File::Copy::Recursive" => 0,
    "File::Temp" => 0,
    "IO::Capture::Stdout" => 0,
    "Module::Pluggable" => 0,
    "Test::File" => 0,
    "Test::MockTime" => 0,
    "Test::More" => 0,
    "Test::Most" => 0,
    "Test::Trap" => 0,
    "lib" => 0
  }
);


my %fallback_build_requires = (
  "File::Copy::Recursive" => 0,
  "File::Temp" => 0,
  "IO::Capture::Stdout" => 0,
  "Module::Build" => "0.28",
  "Module::Pluggable" => 0,
  "Test::File" => 0,
  "Test::MockTime" => 0,
  "Test::More" => 0,
  "Test::Most" => 0,
  "Test::Trap" => 0,
  "lib" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
